package ru.dbotthepony.mc.otm.capability;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.Tag;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraftforge.common.util.INBTSerializable;
import net.minecraftforge.registries.ForgeRegistry;
import net.minecraftforge.registries.RegistryManager;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Objects;
import java.util.UUID;

public record MatterTask(@Nonnull UUID id, @Nullable UUID pattern, @Nonnull Item item, int in_progress, int finished, int required) {
	public MatterTask(@Nonnull UUID id, @Nullable UUID pattern, @Nonnull Item item, int in_progress, int finished, int required) {
		this.id = id;
		this.pattern = pattern;
		this.item = item;
		this.in_progress = Math.max(0, in_progress);
		this.finished = Math.max(0, finished);
		this.required = Math.max(0, required);
	}

	@Override
	public int hashCode() {
		return id.hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof MatterTask obj1)
			return obj1.id.equals(id);

		return false;
	}

	public ItemStack stack() {
		return new ItemStack(item, 1);
	}

	public ItemStack stack(int amount) {
		return new ItemStack(item, amount);
	}

	public int total() {
		return in_progress + finished + required;
	}

	public MatterTask shrinkRequired(int amount) {
		return new MatterTask(id, pattern, item, in_progress + amount, finished, required - amount);
	}

	public MatterTask shrinkInProgress(int amount) {
		return new MatterTask(id, pattern, item, in_progress - amount, finished + amount, required);
	}

	public CompoundTag serializeNBT() {
		CompoundTag tag = new CompoundTag();

		tag.putLong("id_l", id.getLeastSignificantBits());
		tag.putLong("id_u", id.getMostSignificantBits());

		if (pattern != null) {
			tag.putLong("pattern_l", pattern.getLeastSignificantBits());
			tag.putLong("pattern_u", pattern.getMostSignificantBits());
		}

		tag.putString("item", Objects.requireNonNull(item.getRegistryName()).toString());
		tag.putInt("in_progress", in_progress);
		tag.putInt("finished", finished);
		tag.putInt("required", required);

		return tag;
	}

	@Nullable
	public static MatterTask deserializeNBT(@Nullable Tag nbt) {
		if (nbt == null)
			return null;

		if (nbt instanceof CompoundTag tag) {
			Item get_item = RegistryManager.ACTIVE.getRegistry(Item.class).getValue(new ResourceLocation(tag.getString("item")));

			if (get_item != null && get_item != Items.AIR) {
				long a = tag.getLong("pattern_u");
				long b = tag.getLong("pattern_l");

				UUID pattern = a != 0 && b != 0 ? new UUID(a, b) : null;

				return new MatterTask(
					new UUID(tag.getLong("id_u"), tag.getLong("id_l")),
					pattern,
					get_item,
					tag.getInt("in_progress"),
					tag.getInt("finished"),
					tag.getInt("required")
				);
			}
		}

		return null;
	}

	public void write(FriendlyByteBuf buffer) {
		buffer.writeLong(id.getMostSignificantBits());
		buffer.writeLong(id.getLeastSignificantBits());

		if (pattern != null) {
			buffer.writeBoolean(true);

			buffer.writeLong(pattern.getMostSignificantBits());
			buffer.writeLong(pattern.getLeastSignificantBits());
		} else {
			buffer.writeBoolean(false);
		}

		buffer.writeInt(((ForgeRegistry<Item>) RegistryManager.ACTIVE.getRegistry(Item.class)).getID(item));
		buffer.writeInt(in_progress);
		buffer.writeInt(finished);
		buffer.writeInt(required);
	}

	@Nullable
	public static MatterTask read(FriendlyByteBuf buffer) {
		var id = new UUID(buffer.readLong(), buffer.readLong());
		var pattern = buffer.readBoolean() ? new UUID(buffer.readLong(), buffer.readLong()) : null;
		var item = ((ForgeRegistry<Item>) RegistryManager.ACTIVE.getRegistry(Item.class)).getValue(buffer.readInt());

		if (item == null)
			return null;

		var in_progress = buffer.readInt();
		var finished = buffer.readInt();
		var required = buffer.readInt();

		return new MatterTask(id, pattern, item, in_progress, finished, required);
	}
}
